<?php
/**
 * Plugin Name: Social Profiles Sidebar Widget
 * Plugin URI: http://blog.ppfeufer.de/wordpress-plugin-social-profiles-sidebar-widget/
 * Description: Add a sidebarwidget for social profiles in you blog. Supports several sets of icons and three different iconsizes. 32, 64, and 128 Pixel.
 * Version: 1.1.1
 * Author: H.-Peter Pfeufer
 * Author URI: http://ppfeufer.de
 */

/**
 * Supports the following profiles.
 * Delicious
 * Facebook
 * Flickr
 * Google
 * Linkarena
 * Linkedin
 * RSS
 * Technorati
 * Twitter
 * Wikio
 * Wong
 * Xing
 * Yahoo
 * Yigg
 * Youtube
 *
 * All imagefiles have to be in .png-format !!!
 */

/**
 * Changelog
 * = 1.1.1 (25.11.2010) =
 * Cleaned up some directorys
 *
 * = 1.1.0 (25.11.2010) =
 * Added JavaScript to detect width of sidebar and center iconblock. (Looks better now, thanks to Simon).
 * Added JavaScript to show changes in select immediatly (Thanks again to Simon)
 * Added german translation.
 * Added function to check if profile-icon exists (Not every Iconset supports the same or all profiles).
 * Added new iconset "Crumpled Paper Social Media Logos"
 * Added new iconset "DyIcon Socialize Icon Set" by DryIcon
 *
 * = 1.0.3 (24.11.2010) =
 * Corrected misspelling for linkedin-image.
 *
 * = 1.0.2 (24.11.2010) =
 * Updated plugindescription in readme.txt.
 *
 * = 1.0.1 (24.11.2010) =
 * Corrected a little XML-Setting.
 *
 * = 1.0.0 (24.11.2010) =
 * Initial Release.
 * Added javascript for selecting iconsets.
 * Added function to read XML-data from iconsets.
 * *
 */

/**
 * ToDo
 * @todo Add javascript to show iconset after selecting it.
 * @todo Automatische Erkennung der verschiedenen Icongrößen.
 * @todo Automatische Erkennung, welche Icons im Set vorhanden sind.
 * @todo Textlinks neben den Icons bei 16px (Bohncore).
 */

define('SOCIALPROFILESSIDEBARWIDGETVERSION', '1.1.1');

class Social_Profiles_Sidebar extends WP_Widget {
	/**
	 * Einige notwendige Variablen definieren
	 */
	protected $var_sPluginDir = '../wp-content/plugins/social-profiles-sidebar-widget/';
	protected $var_sIconsetDir = '/wp-content/plugins/social-profiles-sidebar-widget/iconsets/';
	protected $var_sWhat = 'dir';
	protected $var_bRecursive = false;

	/**
	 * Constructor
	 */
	function Social_Profiles_Sidebar() {
		if(function_exists('load_plugin_textdomain')) {
			load_plugin_textdomain('social-profiles-sidebar-widget', PLUGINDIR . '/' . dirname(plugin_basename(__FILE__)) . '/languages', dirname(plugin_basename(__FILE__)) . '/languages');
		}

		$widget_ops = array(
			'classname' => 'social_profiles_sidebar',
			'description' => __('The Social Profiles Sidebar', 'social-profiles-sidebar-widget')
		);

		$control_ops = array(
			'width' => 400
		);

		$this->WP_Widget('social_profiles_sidebar', 'The Social Profiles Sidebar', $widget_ops, $control_ops);
	}

	/**
	 * Widgetformular erstellen
	 * @param $instance
	 */
	function form($instance) {
		$instance = wp_parse_args((array) $instance, array(
			'title' => '',
			'selected_iconset' => 'none',
			'iconsize' => 'none',
			'profile' => array(
				'delicious' => '',
				'facebook' => '',
				'flickr' => '',
				'google' => '',
				'linkarena' => '',
				'linkedin' => '',
				'rss' => '',
				'technorati' => '',
				'twitter' => '',
				'wikio' => '',
				'wong' => '',
				'xing' => '',
				'yahoo' => '',
				'yigg' => '',
				'youtube' => ''
			)
		));

		$title = strip_tags($instance['title']);
		$selected_iconset = htmlspecialchars($instance['selected_iconset']);
		$selected_iconsize = htmlspecialchars($instance['iconsize']);

		/**
		 * Verzeichnis nach Iconsets durchsuchen
		 */
		$array_dir = $this->verzeichnis_auslesen('../' . $this->var_sIconsetDir, $this->var_sWhat, $this->var_bRecursive);

		/**
		 * JavaScript einfügen
		 */
		$api_script = $this->var_sPluginDir . 'javascript/flattr.js';
		echo '<script type="text/javascript" src="' . $api_script . '"></script>';

		/**
		 * Wenn Iconsets vorhanden sind
		 */
		if($array_dir) {
			/**
			 * Daten des Iconsets auslesen und in einem Array unterbringen
			 */
			for($count_i = 0; $count_i < count($array_dir); $count_i++) {
				if(file_exists('../' . $this->var_sIconsetDir . $array_dir[$count_i] . '/iconset.xml')) { // Wenn iconset.xml existiert
					$var_xmlUrl = '../' . $this->var_sIconsetDir . $array_dir[$count_i] . '/iconset.xml'; // XML feed file/URL
					$var_xmlStr = file_get_contents($var_xmlUrl);
					$obj_xmlObj = simplexml_load_string($var_xmlStr);

					$array_iconsetData[$array_dir[$count_i]] = $this->XMLToArray($obj_xmlObj);
				}
			}

			// Title
			echo '<p style="border-bottom: 1px solid #DFDFDF;"><strong>' . __('Title:', 'social-profiles-sidebar-widget') . '</strong></p>';
			echo '<p><input id="' . $this->get_field_id('title') . '" name="' . $this->get_field_name('title') . '" type="text" value="' . $instance['title'] . '" size="48" /></p>';

			// Iconset
			echo '<p style="clear:both;"></p>';
			echo '<p style="border-bottom: 1px solid #DFDFDF;"><strong>' . __('Iconset:', 'social-profiles-sidebar-widget') . '</strong></p>';
			echo '<p><small>' . __('Please choose an iconset and type in your profilelinks.', 'social-profiles-sidebar-widget') . '</small></p>';
			echo '<p><select id="' . $this->get_field_id('selected_iconset') . '" name="' . $this->get_field_name('selected_iconset') . '" onchange="wpWidgets.save(jQuery(this).closest(\'div.widget\'),0,1,0);">' . "\n";
			echo '<option value="none">' . __('Your choice', 'social-profiles-sidebar-widget') . '</option>' . "\n";

			if($array_iconsetData && is_array($array_iconsetData)) {
				foreach($array_iconsetData as $key => $value) {
					if($selected_iconset == $key) {
						$selected = ' selected="selected"';
					} else {
						$selected = '';
					}

					echo '<option value="' . $key . '"' . $selected . '>' . $value['icons']['name'] . '</option>' . "\n";
				}
			}

			echo '</select></p>';

			if($selected_iconset == 'none') {
				echo '<p style="background-color:#f00; padding:10px; font-weight:bold;">' . __('Please select an iconset !!!', 'social-profiles-sidebar-widget') . '</p>';
			} else {
				echo '<p style="border-bottom: 1px solid #DFDFDF;"><strong>' . __('Selected iconset:', 'social-profiles-sidebar-widget') . '</strong></p>';
				if($array_iconsetData[$selected_iconset]['icons']['screenshot'] && $array_iconsetData[$selected_iconset]['icons']['screenshot'] != '') {
					echo '<p style="float:left; margin-right:10px;"><img src="' . '../' . $this->var_sIconsetDir . $selected_iconset . '/' . $array_iconsetData[$selected_iconset]['icons']['screenshot'] . '" width="125" height="125" alt="Screenshot: ' . $array_iconsetData[$selected_iconset]['icons']['name'] . '" /></p>';
				}

				if($array_iconsetData[$selected_iconset]['icons']['uri']) {
					echo '<p><strong>' . __('Selected iconset', 'social-profiles-sidebar-widget') . ':</strong><br /><a href="' . $array_iconsetData[$selected_iconset]['icons']['uri'] . '">' . $array_iconsetData[$selected_iconset]['icons']['name'] . '</a></p>';
				} else {
					echo '<p><strong>' . __('Selected iconset', 'social-profiles-sidebar-widget') . ':</strong><br />' . $array_iconsetData[$selected_iconset]['icons']['name'] . '</p>';
				}

				if($array_iconsetData[$selected_iconset]['author']['uri']) {
					echo '<p><strong>' . __('Author', 'social-profiles-sidebar-widget') . ':</strong><br /><a href="' . $array_iconsetData[$selected_iconset]['author']['uri'] . '">' . $array_iconsetData[$selected_iconset]['author']['name'] . '</a></p>';
				} else {
					echo '<p><strong>' . __('Author', 'social-profiles-sidebar-widget') . ':</strong><br />' . $array_iconsetData[$selected_iconset]['author']['name'] . '</p>';
				}

				if($array_iconsetData[$selected_iconset]['icons']['flattr'] == 1) {
					echo '<p><strong>' . __('Like this set? Support the author.', 'social-profiles-sidebar-widget') . '</strong><br /><a class="FlattrButton" style="display:none;" rev="flattr;button:compact;" href="' . $array_iconsetData[$selected_iconset]['icons']['uri'] . '"></a></p>';
				}

				// Supported Profiles
				echo '<p style="clear:both;"></p>';
				echo '<p style="border-bottom: 1px solid #DFDFDF;"><strong>' . __('Supported Profiles:', 'social-profiles-sidebar-widget') . '</strong></p>';
				$array_profileImages = $this->verzeichnis_auslesen('../' . $this->var_sIconsetDir . $selected_iconset . '/32x32/', 'files', $this->var_bRecursive);

				foreach($array_profileImages as $value) {
					echo ucfirst(str_replace('.png', '', $value)) . ' ';
				}

				// Icongrösse
				echo '<p style="clear:both;"></p>';
				echo '<p style="border-bottom: 1px solid #DFDFDF;"><strong>' . __('Iconsize:', 'social-profiles-sidebar-widget') . '</strong></p>';
				echo '<p><select id="' . $this->get_field_id('iconsize') . '" name="' . $this->get_field_name('iconsize') . '" onchange="wpWidgets.save(jQuery(this).closest(\'div.widget\'),0,1,0);">' . "\n";

				$array_EntrysortOtpions = array(
					'none' => array(
						'selection' => ($selected_iconsize === 'none') ? ' selected="selected"' : '',
						'translation' => __('Your choice', 'social-profiles-sidebar-widget')
					),
					'32x32' => array(
						'selection' => ($selected_iconsize === '32x32') ? ' selected="selected"' : '',
						'translation' => __('32 x 32 Pixel', 'social-profiles-sidebar-widget')
					),
					'64x64' => array(
						'selection' => ($selected_iconsize === '64x64') ? ' selected="selected"' : '',
						'translation' => __('64 x 64 Pixel', 'social-profiles-sidebar-widget')
					),
					'128x128' => array(
						'selection' => ($selected_iconsize === '128x128') ? ' selected="selected"' : '',
						'translation' => __('128 x 128 Pixel', 'social-profiles-sidebar-widget')
					)
				);

				foreach($array_EntrysortOtpions as $arraykey => $arrayvalue) {
					echo '<option value="' . $arraykey . '"' . $arrayvalue['selection'] . '>' . $arrayvalue['translation'] . '</option>';
				}

				echo '</select></p>';

				if($selected_iconsize == 'none') {
					echo '<p style="background-color:#f00; padding:10px; font-weight:bold;">' . __('Please select an iconsize !!!', 'social-profiles-sidebar-widget') . '</p>';
				} else {
					// Profilelinks
					echo '<p style="clear:both;"></p>';
					echo '<p style="border-bottom: 1px solid #DFDFDF;"><strong>' . __('My Profiles:', 'social-profiles-sidebar-widget') . '</strong></p>';

					foreach($array_profileImages as $value) {
						$var_sProfilename = ucfirst(str_replace('.png', '', $value));
						$var_sProfileFieldName = str_replace('.png', '', $value);

						echo '<p><span style="display:inline-block; width:100px;">' . $var_sProfilename . ': </span><input id="' . $this->get_field_id($var_sProfileFieldName) . '" name="' . $this->get_field_name($var_sProfileFieldName) . '" type="text" value="' . $instance['profile'][$var_sProfileFieldName] . '" size="35" /></p>';
					}
				}
			}

			// Flattr
			echo '<p style="clear:both;"></p>';
			echo '<p style="border-bottom: 1px solid #DFDFDF;"><strong>' . __('Like this Plugin? Support the developer.', 'social-profiles-sidebar-widget') . '</strong></p>';
			echo '<p><a class="FlattrButton" style="display:none;" rev="flattr;button:compact;" href="http://blog.ppfeufer.de/wordpress-plugin-social-profiles-sidebar-widget/"></a></p>';

			echo '<div id="selected-iconset-data">';
			echo '</div>';
		}

		echo '<p style="clear:both;"></p>';
	}

	/**
	 * Widget erstellen
	 * @param unknown_type $args
	 * @param unknown_type $instance
	 */
	function widget($args, $instance) {
		extract($args);

		echo $before_widget;

		$title = (empty($instance['title'])) ? '&nbsp;' : apply_filters('widget_title', $instance['title']);

		if(!empty($title)) {
			echo $before_title . $title . $after_title;
		}

		echo $this->social_profiles_sidebar_output($instance, 'widget');
		echo $after_widget;
	}

	/**
	 * Optionen updaten
	 * @param $new_instance
	 * @param $old_instance
	 */
	function update($new_instance, $old_instance) {
		$instance = $old_instance;
		print_r($instance);
		$new_instance = wp_parse_args((array) $new_instance, array(
			'title' => '',
			'selected_iconset' => 'none',
			'iconsize' => 'none',
			'profile' => array(
				'delicious' => '',
				'facebook' => '',
				'flickr' => '',
				'google' => '',
				'linkarena' => '',
				'linkedin' => '',
				'rss' => '',
				'technorati' => '',
				'twitter' => '',
				'wikio' => '',
				'wong' => '',
				'xing' => '',
				'yahoo' => '',
				'yigg' => '',
				'youtube' => ''
			)
		));

		$instance['title'] = strip_tags($new_instance['title']);
		$instance['selected_iconset'] = strip_tags($new_instance['selected_iconset']);
		$instance['iconsize'] = strip_tags($new_instance['iconsize']);
		$instance['profile']['delicious'] = strip_tags($new_instance['delicious']);
		$instance['profile']['facebook'] = strip_tags($new_instance['facebook']);
		$instance['profile']['flickr'] = strip_tags($new_instance['flickr']);
		$instance['profile']['google'] = strip_tags($new_instance['google']);
		$instance['profile']['linkarena'] = strip_tags($new_instance['linkarena']);
		$instance['profile']['linkedin'] = strip_tags($new_instance['linkedin']);
		$instance['profile']['rss'] = strip_tags($new_instance['rss']);
		$instance['profile']['technorati'] = strip_tags($new_instance['technorati']);
		$instance['profile']['twitter'] = strip_tags($new_instance['twitter']);
		$instance['profile']['wikio'] = strip_tags($new_instance['wikio']);
		$instance['profile']['wong'] = strip_tags($new_instance['wong']);
		$instance['profile']['xing'] = strip_tags($new_instance['xing']);
		$instance['profile']['yahoo'] = strip_tags($new_instance['yahoo']);
		$instance['profile']['yigg'] = strip_tags($new_instance['yigg']);
		$instance['profile']['youtube'] = strip_tags($new_instance['youtube']);

		return $instance;
	}

	/**
	 * XML parsen
	 *
	 * @param string $x_xml
	 */
	function XMLToArray($arrObjData, $arrSkipIndices = array()) {
		$arrData = array();

		// if input is object, convert into array
		if(is_object($arrObjData)) {
			$arrObjData = get_object_vars($arrObjData);
		}

		if(is_array($arrObjData)) {
			foreach($arrObjData as $index => $value) {
				if(is_object($value) || is_array($value)) {
					$value = $this->XMLToArray($value, $arrSkipIndices); // recursive call
				}

				if(in_array($index, $arrSkipIndices)) {
					continue;
				}

				$arrData[$index] = $value;
			}
		}

		return $arrData;
	}

	/**
	 * Funktion zum Auslesen eines Verzeichnisses
	 *
	 * @param $path string Verzeichnispfad relativ zum aufrufenden Script
	 * @param $what string Verzeichnisse und/oder Dateien? ('all', 'dir', 'file')
	 * @param $rekursiv bool Unterverzeichnisse mit auflisten? ( true, false)
	 *
	 * @return array
	 */
	function verzeichnis_auslesen($path, $what = 'all', $rekursiv = true) {
		$path = preg_replace('~(.*)/$~', '\\1', $path); // letzten Slash aus dem String entfernen
		$list = array();
		$directory = opendir($path);

		while($dir_entry = readdir($directory)) {
			switch($what) {
				case 'file':
					if(is_file("$path/$dir_entry")) $list[] = $rekursiv ? "$path/$dir_entry" : $dir_entry;
					break;

				case 'dir':
					if(is_dir("$path/$dir_entry") && !preg_match('~^\.\.?$~', $dir_entry)) $list[] = $rekursiv ? "$path/$dir_entry" : $dir_entry;
					break;

				default:
					if(!preg_match('~^\.\.?$~', $dir_entry)) $list[] = $rekursiv ? "$path/$dir_entry" : $dir_entry;
			}

			// Unterverzeichnisse?
			if(!preg_match('~^\.\.?$~', $dir_entry) && is_dir("$path/$dir_entry") && $rekursiv) {
				$list2 = $this->verzeichnis_auslesen("$path/$dir_entry", $what, $rekursiv);

				foreach($list2 as $dir_entry2) {
					$list[] = $dir_entry2;
				}
			}
		}

		closedir($directory);
		sort($list);

		return $list;
	}

	function social_profiles_sidebar_output($args = array(), $position) {
		$var_sIconUri = get_bloginfo('home') . $this->var_sIconsetDir . $args['selected_iconset'] . '/' . $args['iconsize'] . '/';
		$var_sIconPath = PLUGINDIR . '/' . dirname(plugin_basename(__FILE__)) . '/iconsets/' . $args['selected_iconset'] . '/' . $args['iconsize'] . '/';

		echo '<div id="social-profiles-widget-wrapper" style="margin-top:10px;">';

		if($args['selected_iconset'] == 'none' || $args['iconsize'] == 'none') {
			echo '<p>' . __('Widget has to be configured properly', 'social-profiles-sidebar-widget') . '</p>';

			if($args['selected_iconset'] == 'none') {
				echo '<p>' . __('No iconset selected', 'social-profiles-sidebar-widget') . '</p>';
			} elseif($args['iconsize'] == 'none') {
				echo '<p>' . __('No iconsize selected', 'social-profiles-sidebar-widget') . '</p>';
			}
		} else {
			foreach($args['profile'] as $key => $value) {
				if($value == '') {
					continue;
				} else {
					if(file_exists($var_sIconPath . $key . '.png')) {
						echo '<span style="display:inline-block; float:left;"><a href="' . $value . '"><img src="' . $var_sIconUri . $key . '.png" alt="' . __('My profile at ' . $key, 'social-profiles-sidebar-widget') . '" title="' . __('My profile at ' . $key, 'social-profiles-sidebar-widget') . '" onload="centerSocialProfilesWidgets();" /></a></span>';
					}
				}
			}
		}

		echo '</div>';

		echo '<script type="text/javascript">' . "\n";
		echo '/* <![CDATA[ */' . "\n";
		echo 'function centerSocialProfilesWidgets() {' . "\n";
		echo '	centerSocialProfilesWidgets = function () {};' . "\n";
		echo '	var wrapperDiv = document.getElementById(\'social-profiles-widget-wrapper\');' . "\n";
		echo '	var sidebarLi = wrapperDiv.parentNode;' . "\n";
		echo '	var liWidth = sidebarLi.clientWidth;' . "\n";
		echo '	var icon = sidebarLi.getElementsByTagName(\'span\')[0];' . "\n";
		echo '	if (icon) {' . "\n";
		echo '		var iconWidth = icon.clientWidth;' . "\n";
		echo '		var padding = Math.floor((liWidth % iconWidth) / 2);' . "\n";
		echo '		wrapperDiv.style.paddingLeft = padding + \'px\';' . "\n";
		echo '	}' . "\n";
		echo '}' . "\n";
		echo '/* ]]> */' . "\n";
		echo '</script>' . "\n";
	}
}

add_action('widgets_init', create_function('', 'return register_widget("Social_Profiles_Sidebar");'));
?>